@@ -4,19 +4,20 @@ import logging
 from itertools import product
 from operator import attrgetter
 from pathlib import Path
+import warnings
 from typing import List, Union
 
-import numpy as np
 import neurom as nm
+import numpy as np
 from neurom import geom, iter_sections, load_neuron
-from neurom.core import Tree
+from neurom.core import Section
 from neurom.core._neuron import Neuron
 from neurom.core.dataformat import COLS
 from scipy.optimize import minimize
 from scipy.special import factorial
 
-from neuror.cut_plane.planes import HalfSpace, PlaneEquation
 from neuror.cut_plane.legacy_detection import internal_cut_detection
+from neuror.cut_plane.planes import HalfSpace, PlaneEquation
 
 L = logging.getLogger(__name__)
 
@@ -101,9 +102,9 @@ class CutPlane(HalfSpace):
         to the coordinate of the first and last point of the neuron.
 
         Description of the algorithm:
+
         1) The distribution of all points along X, Y and Z is computed
            and put into 3 histograms.
-
         2) For each histogram we look at the first and last empty bins
            (ie. the last bin before the histogram starts rising,
            and the first after it reaches zero again). Under the assumption
@@ -132,6 +133,9 @@ class CutPlane(HalfSpace):
         Returns:
             A cut plane object
         """
+        warnings.warn("This method will be deprecated in favor or cut_leaves.find_cut_leaves"
+                      "it also has bugs if one uses +1 in searched_half_spaces and multiple"
+                      "combinations of searched_arguments.", DeprecationWarning)
         if not isinstance(neuron, Neuron):
             neuron = load_neuron(neuron)
 
@@ -184,7 +188,7 @@ class CutPlane(HalfSpace):
         '''Returns sections that ends within the cut plane'''
         leaves = np.array([leaf
                            for neurite in self.morphology.neurites
-                           for leaf in iter_sections(neurite, iterator_type=Tree.ileaf)])
+                           for leaf in iter_sections(neurite, iterator_type=Section.ileaf)])
         leaves_coord = [leaf.points[-1, COLS.XYZ] for leaf in leaves]
         return leaves[self.distance(leaves_coord) < self.bin_width]
 
@@ -195,13 +199,15 @@ class CutPlane(HalfSpace):
 
     def to_json(self):
         '''Return a dictionary with the following items:
-            status: 'ok' if everything went right, else an informative string
-            cut_plane: a tuple (plane, position) where 'plane' is 'X', 'Y' or 'Z'
+
+            - status: 'ok' if everything went right, else an informative string
+            - cut_plane: a tuple (plane, position) where 'plane' is 'X', 'Y' or 'Z'
                        and 'position' is the position
-            cut_leaves: an np.array of all termination points in the cut plane
-            figures: if 'display' option was used, a dict where values are tuples (fig, ax)
+            - cut_leaves: an np.array of all termination points in the cut plane
+            - figures: if 'display' option was used, a dict where values are tuples (fig, ax)
                      for each figure
-            details: A dict currently only containing -LogP of the bin where the cut plane was found
+            - details: A dict currently only containing -LogP of the bin where the cut plane was
+                     found
         '''
         return {'cut-leaves': self.cut_leaves_coordinates,
                 'status': self.status,
