@@ -12,7 +12,7 @@ from nose.tools import assert_dict_equal, assert_equal, ok_
 from numpy.testing import assert_array_almost_equal, assert_array_equal
 
 import neuror.main as test_module
-from neuror.main import Action, Repair
+from neuror.main import Action, Repair, _PARAMS
 from neuror.utils import RepairType
 
 from .expected_sholl_stats import SHOLL_STATS
@@ -159,24 +159,13 @@ def test__get_sholl_proba():
 
 
 def test_continuation():
-    test_module._continuation(DummySection([[1.0, 1, 1, 1], [1, 1, 2, 1]]), [0, 0, 0])
-    test_module._continuation(DummySection([[1.0, 1, 1, 1], [1, 1, 2, 1]]), [1, 1, 1])
 
-
-def test_get_similar_child_diameters():
-    children = [DummySection([[1.0, 1, 1, 10]]),
-                DummySection([[1.0, 1, 1, 12]])]
-    sections = [DummySection([[1.0, 1, 1, 1], [1, 1, 2, 1]], children=children)]
-
-    similar_section = DummySection([[1.0, 1, 1, 1], [1, 1, 2, 1]], children=[])
-    assert_array_equal(test_module._get_similar_child_diameters(sections, similar_section),
-                       [20, 24])
-
-    different_section = DummySection([[1.0, 1, 1, 42]],
-                                     children=[DummySection([[1.0, 1, 1, 1]]),
-                                               DummySection([[1.0, 1, 1, 2]])])
-    assert_array_equal(test_module._get_similar_child_diameters(sections, different_section),
-                       [84, 84])
+    test_module._continuation(
+        DummySection([[1.0, 1, 1, 1], [1, 1, 2, 1]]), [0, 0, 0], _PARAMS, 1., 0.1
+    )
+    test_module._continuation(
+        DummySection([[1.0, 1, 1, 1], [1, 1, 2, 1]]), [1, 1, 1], _PARAMS, 1., 0.1
+    )
 
 
 def test_get_origin():
@@ -200,7 +189,8 @@ def test_get_order_offset():
 
 def test__get_sholl_layer():
     section = SIMPLE.neurites[0].root_node
-    assert_equal(test_module._get_sholl_layer(section, SIMPLE.soma.center), 0)
+    assert_equal(test_module._get_sholl_layer(
+        section, SIMPLE.soma.center, _PARAMS['sholl_layer_size']), 0)
 
 
 def test_last_segment_vector():
@@ -216,12 +206,13 @@ def test__grow_until_sholl_sphere():
     np.random.seed(0)
     neuron = load_neuron(DATA_PATH / 'simple.swc')
     section = neuron.neurites[0].root_node
-    test_module._grow_until_sholl_sphere(section, SIMPLE.soma.center, 0)
+    test_module._grow_until_sholl_sphere(section, SIMPLE.soma.center, 0, _PARAMS,
+                                         lambda diam:diam, 0.1, 1.0)
     assert_array_almost_equal(section.points[:, COLS.XYZ],
-                              np.array([[0., 0., 0.],
-                                        [0., 5., 0.],
-                                        [0.170201, 9.98424, 0.358311],
-                                        [0.469826, 14.901759, 1.211672]], dtype=np.float32))
+                              np.array([[ 0.      ,  0.      ,  0.      ],
+                                        [ 0.      ,  5.      ,  0.      ],
+                                        [ 0.302276,  9.950119,  0.636358],
+                                        [ 1.172574, 14.19536 ,  3.130453]], dtype=np.float32))
 
 
 def test__compute_sholl_data():
@@ -243,6 +234,14 @@ def test__compute_sholl_data():
                                Action.TERMINATION: 2}}})
 
 
+def test__external_apical_point():
+    obj = Repair(DATA_PATH / 'repair_type.asc', apical_point=[1.0, 0.0, 0.0])
+    assert_array_almost_equal(obj.apical_section.points,
+                              np.array([[0., 0., 4., 1.],
+                                        [0., 0., 0., 0.5],
+                                        [1., 0., 0., 0.5]], dtype=np.float32))
+
+
 def test__fill_repair_type_map():
     obj = Repair(DATA_PATH / 'repair_type.asc')
     obj.apical_section = obj.neuron.sections[3]
@@ -306,10 +305,11 @@ def test__grow():
         'dendritic_sections': obj.neuron.sections,
         'intact_branching_angles': {RepairType.basal: {1: [0.2]}}
     }
-
+    obj.current_trunk_radius = 1.0
     obj._grow(leaf, 0, obj.neuron.soma.center)
     assert_equal(len(obj.neuron.sections), 8)
 
+
 def test_repair_axon():
     filename = DATA_PATH / 'real-with-axon.asc'
     with TemporaryDirectory('test-cli-axon') as tmp_folder:
@@ -317,22 +317,22 @@ def test_repair_axon():
         test_module.repair(filename, outfilename, axons=[filename])
         neuron_in = load_neuron(filename)
         neuron_out = load_neuron(outfilename)
-        axon = neuron_out.section(40)
+        axon = neuron_out.section(41)
         ok_(axon.type == NeuriteType.axon)
         assert_array_equal(neuron_in.section(40).points[0],
                            neuron_out.section(40).points[0])
-        ok_(len(neuron_out.section(40).points) > len(neuron_in.section(40).points))
+        ok_(len(neuron_out.section(41).points) > len(neuron_in.section(41).points))
 
         # Test disactivating the axon repair
         repair_flags = {RepairType.axon: False}
         test_module.repair(filename, outfilename, axons=[filename], repair_flags=repair_flags)
         neuron_out = load_neuron(outfilename)
-        axon = neuron_out.section(40)
+        axon = neuron_out.section(41)
         ok_(axon.type == NeuriteType.axon)
-        assert_array_equal(neuron_in.section(40).points[0],
-                           neuron_out.section(40).points[0])
+        assert_array_equal(neuron_in.section(41).points[0],
+                           neuron_out.section(41).points[0])
 
-        ok_(len(neuron_out.section(40).points) == len(neuron_in.section(40).points),
+        ok_(len(neuron_out.section(41).points) == len(neuron_in.section(41).points),
             'The section should not have been regrown')
 
 
