@@ -3,11 +3,13 @@ from tempfile import TemporaryDirectory
 
 import numpy as np
 import pandas as pd
+from morph_tool import diff
 from neurom import load_neuron
+from nose.tools import assert_false
 from numpy.testing import assert_array_almost_equal, assert_array_equal
 
-from neuror.cut_plane.detection import CutPlane
 import neuror.unravel as test_module
+from neuror.cut_plane.detection import CutPlane
 
 DATA = Path(__file__).parent / 'data'
 
@@ -25,9 +27,39 @@ def test_get_principal_direction():
                                                                     [10, -1, 0],
                                                                     [10, 1, 0]]),
                               np.array([1, 0, 0]))
-
 def test_unravel():
     neuron, mapping = test_module.unravel(DATA / 'simple.asc')
+    assert_array_almost_equal(neuron.root_sections[0].points,
+                              np.array([[ 0.      ,  0.      ,  0.      ],
+                                        [1.      , 1.      , 0.      ],
+                                        [2.      , 2.      , 0.      ],
+                                        [2.707107, 1.292893, 0.      ]]))
+
+    assert_array_almost_equal(neuron.root_sections[0].children[0].points[0],
+                              np.array([2.707107, 1.292893, 0.    ]))
+
+    assert_array_almost_equal(mapping[['x0', 'y0', 'z0']].values,
+                              [[0., 0., 0.],
+                               [1., 1., 0.],
+                               [2., 0., 0.],
+                               [3., 0., 0.],
+                               [3., 0., 0.],
+                               [4., 1., 0.],
+                               [3., 0., 0.],
+                               [6., 4., 2.]])
+    assert_array_almost_equal(mapping[['x1', 'y1', 'z1']].values,
+                              [[0.        , 0.        , 0.        ],
+                               [0.99999994, 0.99999994, 0.        ],
+                               [1.99999988, 1.99999988, 0.        ],
+                               [2.70710659, 1.29289317, 0.        ],
+                               [2.70710659, 1.29289317, 0.        ],
+                               [3.70710659, 2.29289317, 0.        ],
+                               [2.70710659, 1.29289317, 0.        ],
+                               [5.70710659, 5.29289341, 2.        ]])
+
+
+def test_unravel_no_path_length():
+    neuron, mapping = test_module.unravel(DATA / 'simple.asc', use_path_length=False)
     assert_array_almost_equal(neuron.root_sections[0].points,
                               np.array([[ 0.      ,  0.      ,  0.      ],
                                         [ 1.404784, -0.163042,  0.      ],
@@ -58,6 +90,21 @@ def test_unravel():
                                [6.80289936,  3.55862665, 2.]])
 
 
+def test_unravel_with_backward_segment():
+    '''Test the fix to the  unravel issue
+    which was not working when there was a segment going backward wrt to the window direction
+    (direction from the window first to the window last point)
+    '''
+    neuron, mapping = test_module.unravel(DATA / 'simple-with-backward-segment.asc')
+    assert_array_almost_equal(neuron.root_sections[0].points,
+                              np.array([[0.        , 0.        , 0.        ],
+                                        [0.99999994, 0.99999994, 0.        ],
+                                        [1.9999999 , 1.9999999 , 0.        ],
+                                        [3.4142134 , 3.4142134 , 0.        ],
+                                        [4.828427  , 4.828427  , 0.        ],
+                                        [5.5355334 , 4.1213202 , 0.        ]], dtype=np.float32))
+
+
 def test_unravel_plane():
     with TemporaryDirectory('test-unravel-plane'):
         mapping = pd.read_csv(DATA / 'mapping.csv')
@@ -96,3 +143,10 @@ def test_unravel_all():
 
         test_module.unravel_all(input, output, raw_planes, unravel_planes)
         assert_array_equal(list(output.rglob('*.h5')), [output / 'Neuron_slice.h5'])
+
+def test_legacy():
+    actual, _ = test_module.unravel(DATA / 'legacy-unravel/1-pt-soma.swc', legacy_behavior=True)
+    assert_false(diff(actual, DATA / 'legacy-unravel/expected-1-pt-soma.h5'))
+
+    actual, _ = test_module.unravel(DATA / 'legacy-unravel/3-pts-soma.swc', legacy_behavior=True)
+    assert_false(diff(actual, DATA / 'legacy-unravel/expected-3-pts-soma.h5'))
